#!/usr/bin/env python
# -*- coding: utf-8 -*-

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html

## About Potrace older version: Plugin-potrace.py https://www.gimpscripts.net/2020/09/plugin-potracepy-ver-05.html

# ------------
#| Change Log |
# ------------
# POTRACE - transform bitmaps into vector graphics https://potrace.sourceforge.net/
# Live Preview (gtk2) according to the idea of Tin Tran http://gimpchat.com/viewtopic.php?f=9&t=9586#p284974
# Ver 1.0 25-02-2024 by MareroQ: Initial version.
# Ver 1.1 27-02-2024 by MareroQ: added FG Color SVG 
# Ver 1.2 28-02-2024 by MareroQ: added option - Open SVG file as new image:"None","Many, Individual","One, Merged","Interactive"

from gimpfu import *
import subprocess
import sys, os, gettext
#from shutil import copyfile
import gtk, gimpui
gimpui.gimp_ui_init ()  # add MrQ

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def sys_file(f_name):
	sys_encoded = sys.getfilesystemencoding()
	if os.name == 'nt': coded = f_name.encode(sys_encoded)
	else: coded = f_name
	return(coded)
  
whoiam = os.path.abspath(sys.argv[0])
# initialize internationalisation in a user-locale (by rob_brz1)
locale_directory = os.path.join(os.path.dirname(whoiam), 'locale')
gettext.install( "plugin-potrace", locale_directory, unicode=True )


def rgb_to_hex(rgb_color):
	r = rgb_color[0]
	g = rgb_color[1]
	b = rgb_color[2]
	return '#%02x%02x%02x' % (r,g,b)
# --color #rrggbb        - set foreground color (default black #000000)  
def color(colorParam):
	params = " --color " + str(colorParam)
	return params

#  --turdsize
def turdsizeArg(turdsizeParam):
	params = " -t" + str(turdsizeParam)
	return params
  
def alphamaxArg(alphamaxParam):
	params = " -a" + " " + str(alphamaxParam) + " -s"
	return params
	
def opttoleranceArg(opttoleranceParam):
	params = " -O " + str(opttoleranceParam)
	return params	
  
def potrace(image, layer, filename, rgb_color, turdsizeParam, alphamaxParam, opttoleranceParam, edge_w, edge_c, open_svg):

	colorParam=rgb_to_hex(rgb_color)
	pdb.gimp_layer_resize_to_image_size(layer)
	pdb.gimp_image_resize_to_layers(image)
	active_layer = pdb.gimp_image_get_active_layer(image)
	pdb.gimp_image_set_active_layer(image, active_layer)	
	pdb.gimp_image_raise_layer_to_top(image, active_layer)	
	layer=active_layer	

	
	outputFolder=gimp.directory + os.sep + "tmp"
	if os.access(sys_file(outputFolder), os.W_OK):	

		tempfilename = outputFolder + os.sep + filename + ".bmp"

		pdb.gimp_image_set_active_layer(image, layer)
		tempsvgfilename = outputFolder + os.sep + filename + ".svg"

		#image = pdb.gimp_image_duplicate(image)
	if not image:
		raise RuntimeError

	tempdrawable = pdb.gimp_image_get_active_drawable(image)

	pdb.gimp_progress_set_text ("Saving a copy")
	layer.name = layer.name.decode("utf-8")

	xchange_utf = tempfilename
		#gimp.pdb.file_bmp_save(image, tempdrawable, xchange_utf, xchange_utf, run_mode=RUN_INTERACTIVE)
	pdb.file_bmp_save2(image, tempdrawable, xchange_utf, xchange_utf, 0,1,3)

	try:
		outfile = open(tempsvgfilename, 'w')
	except:
		raise RuntimeError, "Could not open temporary svg file: " + tempsvgfilename
		
# In Linux binDir = "/usr/bin/potrace" or "/usr/local/bin/potrace" ???   
	if os.name == 'nt':
		binDir="potrace.exe"
	else:
		binDir="/usr/bin/potrace"
	resolutionx, resolutiony = pdb.gimp_image_get_resolution(image) 
	command = binDir \
	+ color(colorParam) \
	+ " -r" + str(resolutionx) \
	+ turdsizeArg(turdsizeParam) \
	+ alphamaxArg(alphamaxParam) \
	+ opttoleranceArg(opttoleranceParam) \
	+ " " + "--output" \
	+ " " + tempsvgfilename \
	+ " " + xchange_utf

	print command, tempsvgfilename

	pdb.gimp_progress_set_text ("Tracing")
	pdb.gimp_progress_pulse()

	child = subprocess.Popen(command, shell=True, stdout=outfile)
	child.communicate() # wait for child to terminate

	if os.path.getsize(tempsvgfilename) > 600100 and pathParam == 1:  
		# Magic number, wag for file size as of 2009, my computer, etc.
		pdb.gimp_message("Trace plugin error: SVG file has too many individual paths to import.")
		return  # Leaves files in gimp/tmp, to get cleaned later 
     
	potrace=pdb.gimp_vectors_import_from_file(image, tempsvgfilename, 1, 1)
	image.vectors[0].name= "Potrace"
	
	if edge_w == 0:	

		pdb.gimp_vectors_set_visible(image.vectors[0], True)

	if edge_w > 0:
		pdb.gimp_vectors_set_visible(image.vectors[0], False)
		newLayer = pdb.gimp_layer_new(image, layer.width, layer.height, 1, "Potrace", 100, 0)
		pdb.gimp_image_insert_layer(image, newLayer, None, 0)  
		pdb.gimp_context_set_stroke_method(0) 
		pdb.gimp_context_set_line_width(edge_w)
		pdb.gimp_context_set_foreground(edge_c)
		pdb.gimp_drawable_edit_stroke_item(newLayer, image.vectors[0])  

	#pdb.file_svg_load(tempsvgfilename, "", resolutionx, layer.width, layer.height, 2)	
	os.remove(xchange_utf)

	if open_svg==3:
		svgimage = pdb.file_svg_load(tempsvgfilename, "", layer.width, layer.height, 0, 1, run_mode=RUN_INTERACTIVE)
	if open_svg==1:  
		svgimage = pdb.file_svg_load(tempsvgfilename, "", resolutionx, layer.width, layer.height, 1)
	if open_svg==2:  
		svgimage = pdb.file_svg_load(tempsvgfilename, "", resolutionx, layer.width, layer.height, 2)
	if open_svg!=0:		
		gimp.Display(svgimage)
	gimp.displays_flush()


###########################################################################################################################################################
###########################################################################################################################################################

pattern_ = "filename" 
proceed_with_changes = False

image = 0 #we'll set these when dialog() is called so that we can access them later
drawable = 0
has_preview = False
preview_layer = 0

rgb_color=0
edge_c=0
global_param1 = 2   # turdsizeParam
global_param3 = 1	# alphamaxParam,
global_param5 = 2	# opttoleranceParam
global_param7 = 0

def apply_effect(layer): #function to do work on either preview layer or actual drawable when user clicks OK
    global image


    turdsizeParam = global_param1
    alphamaxParam = global_param3	
    opttoleranceParam = global_param5	
	
    edge_w = global_param7

	
    for layer in image.layers:
        if layer.name=="Potrace":
            pdb.gimp_image_remove_layer(image,pdb.gimp_image_get_layer_by_name(image,"Potrace"))	
    for vector in image.vectors:
        if vector.name=="Potrace":
            pdb.gimp_image_remove_vectors(image,pdb.gimp_image_get_vectors_by_name(image,"Potrace"))
			
    if selected_option_2== "None" :
        open_svg=0	
    if selected_option_2== "Many, Individual" :
        open_svg=1
    if selected_option_2== "One, Merged" :
        open_svg=2
    if selected_option_2== "Interactive" :
        open_svg=3				
		
    #active_layer = pdb.gimp_image_get_active_layer(image)
    #pdb.gimp_image_raise_layer_to_top(image, active_layer)	




    #if proceed_with_changes == 1:
    potrace(image, layer,str(pattern_), rgb_color, turdsizeParam, alphamaxParam, opttoleranceParam, edge_w, edge_c, open_svg)	
        #python_fu_potrace_LP(image, layer, filename, turdsizeParam, alphamaxParam, opttoleranceParam, edge_w, edge_c)

	
    pdb.gimp_displays_flush()
def apply_final(layer): #wrapper to apply effect on final and remove preview_layer meant to be called by on_ok_button_clicked

    gimp.displays_flush()

# Function to update the live preview
		
def update_live_preview(): #this is called everytime some parameter changes
    global  global_param1, global_param2, global_param3, global_param4, global_param5, global_param6, global_param7, global_param8, global_param9
    global image,drawable, vector
    global has_preview,preview_layer #deal with preview layer
 
    if proceed_with_changes==1:
       apply_effect(image.layers[0])
    else: pass	
	
    gimp.displays_flush()	

   
def dialog(image_, drawable_):
    global image, drawable
    #save these for updates
    image = image_
    pdb.gimp_image_undo_disable(image) #for speed and also when user undo it doesn't see our preview creations/deletions
    drawable = drawable_ 

    dialog = gtk.Dialog("Potrace with Live Preview. Ver.1.2.", None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
    dialog.set_default_size(300, 100)
    if edge_c==0:
        global edge_c
        edge_c = (30,30,230)  # Set default color
		
    if rgb_color==0:
        global rgb_color
        rgb_color= (0,0,0) # Set default color
		
    #------------------------------------------------------------
	
    hbox100 = gtk.HBox()
    dialog.vbox.pack_start(hbox100, expand=True, fill=True)
    pattern_label = gtk.Label(" Create SVG file from active layer. Requires potrace files\n      which must be installed in the bin Gimp directory.\n SvG file (for name do not use spaces & special characters)\n      is saved in your home directory in the 'tmp' folder.\n")	
    hbox100.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #------------------------------------------------------------

    hbox5 = gtk.HBox()
    dialog.vbox.pack_start(hbox5, expand=True, fill=True)

    # Create a label for the integer input
    label5 = gtk.Label(" Name SVG file:")
    hbox5.pack_start(label5, expand=False, fill=False, padding=5)

    # Create a text entry for the integer input
    entry5 = gtk.Entry()
    entry5.set_text("filename")
    entry5.set_width_chars(15)  # Adjust the width of input box as needed
    entry5.connect("changed", on_pattern_changed)
    hbox5.pack_start(entry5, expand=False, fill=False, padding=5)
    #-------------------------------------------------------------
	
    label8 = gtk.Label("FG Color (svg):")
    hbox5.pack_start(label8, expand=False, fill=False, padding=1)
	
    color_button = gtk.ColorButton()
    color_button.set_use_alpha(False)  # Set to True if you want to include alpha channel
    color_button.set_color(gtk.gdk.Color(rgb_color[0]*256, rgb_color[1]*256, rgb_color[2]*256))
    color_button.connect("color-set", on_color_changed_rgb)
    hbox5.pack_start(color_button, expand=False, fill=False, padding=5)	
	
    #==========================================================================
    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)
		
    #-------------------------------------------------------------

    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("Turdsize [default=2] - suppress speckles of up to this size:")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------
	
    hbox1 = gtk.HBox()
    dialog.vbox.pack_start(hbox1, expand=True, fill=True)
    label1 = gtk.Label(" ")
    hbox1.pack_start(label1, expand=False, fill=False, padding=5)
    adjustment1 = gtk.Adjustment(value=2, lower=0, upper=50, step_incr=0.1, page_incr=0)
    param1_scale = gtk.HScale(adjustment=adjustment1)
    param1_scale.set_digits(1)  # Display only integers
    hbox1.pack_start(param1_scale, expand=True, fill=True, padding=5)
    label1 = gtk.SpinButton(adjustment1,0, 1 )
    hbox1.pack_start(label1, expand=False, fill=False, padding=5)
    param1_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######	
    param1_scale.connect("value-changed", on_param1_changed)

    #-------------------------------------------------------------

    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------	
    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("Alphamax [default=1] - corner threshold: ")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
   #-------------------------------------------------------------	

    hbox3 = gtk.HBox()
    dialog.vbox.pack_start(hbox3, expand=True, fill=True)
    label3 = gtk.Label("")
    hbox3.pack_start(label3, expand=False, fill=False, padding=5)
    adjustment3 = gtk.Adjustment(value=1, lower=0, upper=30, step_incr=0.05, page_incr=0)
    param3_scale = gtk.HScale(adjustment=adjustment3)
    param3_scale.set_digits(2)  # Display only integers
    hbox3.pack_start(param3_scale, expand=True, fill=True, padding=5)
    label3 = gtk.SpinButton(adjustment3,0, 2 )
    hbox3.pack_start(label3, expand=False, fill=False, padding=5)
    param3_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param3_scale.connect("value-changed", on_param3_changed)

    #-------------------------------------------------------------

    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------	

    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("Opttolerance [default=0.2] - curve optimization tolerance:")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------		

    hbox5 = gtk.HBox()
    dialog.vbox.pack_start(hbox5, expand=True, fill=True)
    label5 = gtk.Label("")
    hbox5.pack_start(label5, expand=False, fill=False, padding=5)
    adjustment5 = gtk.Adjustment(value=0.2, lower=0.05, upper=2.00, step_incr=0.05, page_incr=0)
    param5_scale = gtk.HScale(adjustment=adjustment5)
    param5_scale.set_digits(2)  # Display only integers
    hbox5.pack_start(param5_scale, expand=True, fill=True, padding=5)
    label5 = gtk.SpinButton(adjustment5,0, 2 )
    hbox5.pack_start(label5, expand=False, fill=False, padding=5)
    param5_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param5_scale.connect("value-changed", on_param5_changed)

    #-------------------------------------------------------------
	
    hbox9 = gtk.HBox()
    dialog.vbox.pack_start(hbox9, expand=True, fill=True)	
    hbox2 = gtk.HBox()
    dialog.vbox.pack_start(hbox2, expand=True, fill=True)
    toggle_button = gtk.ToggleButton()
    dialog.action_area.add(toggle_button)	
    update_toggle_button_label(toggle_button)
    toggle_button.connect("toggled", on_toggle_button_toggled)
    hbox9.pack_start(toggle_button, expand=False, fill=False, padding=5)
	
    #-------------------------------------------------------------

    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------

    hbox7 = gtk.HBox()
    dialog.vbox.pack_start(hbox7, expand=True, fill=True)
    label7 = gtk.Label("Stroke Path Width:")
    hbox7.pack_start(label7, expand=False, fill=False, padding=5)
    adjustment7 = gtk.Adjustment(value=0, lower=0, upper=5, step_incr=1, page_incr=0)
    param7_scale = gtk.HScale(adjustment=adjustment7)
    param7_scale.set_digits(0)  # Display only integers
    #hbox7.pack_start(param7_scale, expand=True, fill=True, padding=5)
    label7 = gtk.SpinButton(adjustment7,0, 0 )
    hbox7.pack_start(label7, expand=False, fill=False, padding=5)
    param7_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param7_scale.connect("value-changed", on_param7_changed)
		
    # Create a label for the color picker
    label8 = gtk.Label("        Edge color:")
    hbox7.pack_start(label8, expand=False, fill=False, padding=1)

    # Create a color picker button
    color_button = gtk.ColorButton()
    color_button.set_use_alpha(False)  # Set to True if you want to include alpha channel
    color_button.set_color(gtk.gdk.Color(edge_c[0]*256, edge_c[1]*256, edge_c[2]*256))
    color_button.connect("color-set", on_color_changed_c)
    hbox7.pack_start(color_button, expand=False, fill=False, padding=5)

    #-------------------------------------------------------------	

    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------
    # Combo box	
    hbox6 = gtk.HBox()
    dialog.vbox.pack_start(hbox6, expand=True, fill=True)
    label6 = gtk.Label("Open SVG file as new image:      ")
    hbox6.pack_start(label6, expand=False, fill=False, padding=5)   

    # Create a ComboBox
    combo_box = gtk.ComboBox()
    hbox6.pack_start(combo_box, expand=False, fill=False, padding=5)

    # Create a ListStore model for ComboBox
    list_store = gtk.ListStore(str)
    options = ["None","Many, Individual","One, Merged","Interactive"] # Add your options here
    for option in options:
        list_store.append([option])

    # Set the model for ComboBox
    combo_box.set_model(list_store)

    # Create a CellRendererText to render the text in the ComboBox
    cell_renderer = gtk.CellRendererText()

    # Pack the CellRendererText into the ComboBox
    combo_box.pack_start(cell_renderer, True)
    combo_box.add_attribute(cell_renderer, 'text', 0)

    # Set up the "changed" signal handler   
    combo_box.connect("changed", on_combobox_changed_2)

    # Set the default option
    default_option = "None"
    default_iter = list_store.get_iter_first()
    while default_iter is not None:
        if list_store.get_value(default_iter, 0) == default_option:
            combo_box.set_active_iter(default_iter)
            break
        default_iter = list_store.iter_next(default_iter)
    #-------------------------------------------------------------	
    #===========================================================================
    # Add an CANCEL button
    cancel_buton = dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
    cancel_buton.connect("clicked", cancel)
	
    # Add an OK button
    ok_button = dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
    ok_button.connect("clicked", on_ok_button_clicked)

	
    dialog.show_all()
    dialog.run()
    pdb.gimp_displays_flush()



def on_sample_integer_changed(entry):
    global sample_integer
    try:
        sample_integer = float(entry.get_text()) #int(entry.get_text())
        #pdb.gimp_message(sample_integer)
        update_live_preview()
    except ValueError:
        # Handle the case where the input is not a valid integer
        pass   

def on_pattern_changed(entry):
    global pattern_
    try:
        pattern_ = str(entry.get_text()) #int(entry.get_text())
        #pdb.gimp_message(pattern_)
        update_live_preview()
    except ValueError:
        # Handle the case where the input is not a valid integer
        pass 

def on_color_changed_rgb(widget, data=None):
    global rgb_color
    color = widget.get_color()
    rgb_color = (int(color.red/256), int(color.green/256), int(color.blue/256))
    update_live_preview()
    pdb.gimp_displays_flush()
	
def on_color_changed_c(widget, data=None):
    global edge_c
    color = widget.get_color()
    edge_c = (int(color.red/256), int(color.green/256), int(color.blue/256))
    update_live_preview()
    pdb.gimp_displays_flush()
#######################################################
def on_combobox_changed_2(combobox):
    global selected_option_2
    model = combobox.get_model()
    active_iter = combobox.get_active_iter()

    if active_iter:
        selected_option_2 = model.get_value(active_iter, 0)
    update_live_preview()
	
def on_toggle_button_toggled(button):
    global proceed_with_changes
    proceed_with_changes = not proceed_with_changes
    update_toggle_button_label(button)

def update_toggle_button_label(button):
    label = "Preview = Yes\n*Click to Stop" if proceed_with_changes else "Preview = No \n*Click to Start"
    button.set_label(label)	
    update_live_preview()

def on_combobox_changed(combobox):
    global selected_option
    model = combobox.get_model()
    active_iter = combobox.get_active_iter()

    if active_iter:
        selected_option = model.get_value(active_iter, 0)
    update_live_preview()

def on_combobox_changed_2(combobox):
    global selected_option_2
    model = combobox.get_model()
    active_iter = combobox.get_active_iter()

    if active_iter:
        selected_option_2 = model.get_value(active_iter, 0)
    update_live_preview()
	
def cancel(cancel_buton, data=None):
    global preview_layer
    #pdb.gimp_image_remove_layer(image,preview_layer)
    for layer in image.layers:
        if layer.name=="Potrace":
            pdb.gimp_image_remove_layer(image,pdb.gimp_image_get_layer_by_name(image,"Potrace"))
    for vector in image.vectors:
        if vector.name=="Potrace":			
            pdb.gimp_image_remove_vectors(image,pdb.gimp_image_get_vectors_by_name(image,"Potrace"))			
def on_param1_changed(scale):
    global global_param1
    global_param1 = scale.get_value()
    update_live_preview()
    gimp.displays_flush()

def on_param3_changed(scale):
    global global_param3
    global_param3 = scale.get_value()
    update_live_preview()
    gimp.displays_flush()
	
def on_param5_changed(scale):
    global global_param5
    global_param5 = scale.get_value()
    update_live_preview()
    gimp.displays_flush()
	
def on_param7_changed(scale):
    global global_param7
    global_param7 = scale.get_value()
    update_live_preview()
	
def on_ok_button_clicked(button, data=None):
    global drawable
    #apply_final(drawable)
    button.get_toplevel().destroy() #destroys the gtk dialog window
    #update_live_preview()
#######################################################

register(
    "python_fu_potrace_live_preview",
    "Live Preview Plugin",
    "Create a live preview of an effect autotrace",
    "Lloyd Konneker/TT/MrQ",
    "GPL3",
    "2023",
    "<Image>/Filters/Edge-Detect/Potrace with Live Preview...",  # Menu location
    "*",  # Image type
    [],
    [],
    dialog
)

main()

